#!/usr/bin/perl -w
# logtable.pl (part of Logwrangler) <http://www.reppep.com/~pepper/analog/logwrangler/>
# Usage: logtable.pl stats1 stats2 ... > outfile.html
# logtable.pl generates an HTML table summarizing statistics from multiple COMPUTER format reports generated by Analog 5.
use strict;
use POSIX;
my $version = "2.0";

my ($site, $sitepath, $k);
my %stat;

print
'<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
        "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
	<title>Logwrangler Statistics Summary</title>
</head>
<body bgcolor="white">

<h2 align="center">Logwrangler Statistics Summary</h2>

';

print
"<table>
	<tr>
		<th>site</th>
		<th>from - to</th>
		<th>Total kbytes transferred</th>
		<th>Successful page requests</th>
		<th>Successful requests</th>
		<th>Failed requests</th>
		<th>Redirected requests</th>
		<th>Distinct files requested</th>
		<th>Distinct hosts served</th>
	</tr>";

foreach $site (@ARGV) {
	if ($site =~ m{^(.+/)?([-a-z0-9.]+)}) { $sitepath = $2; }
	else { die "$0: regexp failed -- bad filename $sitepath? "; }
	if ($sitepath =~ m{^([-a-z0-9.]+)\.(txt|out|dat)}) { $sitepath = $1; }
	foreach $k ("B7", "BT", "CL", "F7", "FL", "FRd", "FRh", "FRmi", "FRmo", "FRy", "H7", "LRd", "LRh", "LRmi", "LRmo", "LRy", "N7", "NF", "NH", "P7", "PR", "R7", "RR", "S7", "SR", "UL") { $stat{$k} = 0; }
	$stat{HN} = "";
		# Analog doesn't necessarily create lines for all values
	open (INPUT, $site) or die "failed to open input file $site: $!";
	while (<INPUT>) {
		chomp;
		if (m{^x	FR	(\d+)\t(\d+)\t(\d+)\t(\d+)\t(\d+)$}) {
			($stat{FRy}, $stat{FRmo}, $stat{FRd}, $stat{FRh}, $stat{FRmi}) = ($1, $2, $3, $4, $5)
		}
		elsif (m{^x	LR	(\d+)\t(\d+)\t(\d+)\t(\d+)\t(\d+)$}) {
			($stat{LRy}, $stat{LRmo}, $stat{LRd}, $stat{LRh}, $stat{LRmi}) = ($1, $2, $3, $4, $5)
		}
		elsif (m{^x	([A-Z][A-Z7])	([^/]+)$}) { $stat{$1} = $2; }
	}
	# print "\n$sitepath\n"; 
	# foreach $k (keys %stat) { print "$k: $stat{$k}, "; }
	# print "\n";
	# uncomment these for diagnostics
	print "<tr>\n";
	print "\t\t<td valign=\"middle\"><a href=\"", $sitepath, "/\">", $stat{HN}, "</a></td>\n";
	printf "\t\t<td align=\"left\">%04d/%02d/%02d-%02d:%02d - %04d/%02d/%02d-%02d:%02d</td>\n", $stat{FRy}, $stat{FRmo}, $stat{FRd}, $stat{FRh}, $stat{FRmi}, $stat{LRy}, $stat{LRmo}, $stat{LRd}, $stat{LRh}, $stat{LRmi};
	print "\t\t<td align=\"right\">", commify(ceil($stat{BT} / 1024)), "<br />(", commify(ceil($stat{B7} / 1024)), ")</td>\n";
	print "\t\t<td>", commify($stat{PR}), " (", commify($stat{P7}), ")</td>\n";
	print "\t\t<td>", commify($stat{SR}), " (", commify($stat{S7}), ")</td>\n";
	print "\t\t<td>", commify($stat{FL}), " (", commify($stat{F7}), ")</td>\n";
	print "\t\t<td>", commify($stat{RR}), " (", commify($stat{R7}), ")</td>\n";
	print "\t\t<td>", commify($stat{NF}), " (", commify($stat{N7}), ")</td>\n";
	print "\t\t<td>", commify($stat{NH}), " (", commify($stat{H7}), ")</td>\n";
	print "\t</tr>";
}

print "\n</table>\n";

print
'
<p>Numbers in parentheses are for the last 7 days.</p>

<p>This page was generated by <a href="http://www.reppep.com/~pepper/analog/logwrangler/">http://www.reppep.com/~pepper/analog/logwrangler/</a>, from <a href="http://www.analog.cx/">Analog</a> output.</p>

<hr />

<h5 align="center"><a href="../">Up</a></h5>

</body>
</html>
';


sub commify {
	local $_  = shift;
	1 while s/^([-+]?\d+)(\d{3})/$1,$2/;
	return $_;
}
