#!/usr/bin/perl -w
# logwrangler.pl <http://www.reppep.com/~pepper/analog/logwrangler/>
# "Usage: logwrangler.pl [-Vqv] [--version]";
# Generate multiple reports with Analog and Report Magic, and tabulate results
# Set variables for your local configuration here
use strict;
my $version = "2.0";

my $basedir = glob("~/logwrangler");
# logwrangler parent directory; glob is required to handle '~'
my $dnstran = "/usr/local/dnstran1.5.2/dnstran.app";
# DNSTran binary (optional); this example is for Mac OS X
my @dnstranargs = ("-settings", "/usr/local/dnstran1.5.2/dnstran.cfg");
# DNSTran arguments (optional); this example is for Mac OS X
my $logs = "/var/log/httpd/*access*";
# logs for DNSTran to process
my $analog = ("/usr/local/bin/analog");
# analog binary
my $configpath = "$basedir/cfg";
# directory containing Analog configuration files
my $rmagic = "/usr/local/reportmagic/rmagic.pl";
# path to rmagic.pl
my $datapath = "$basedir/data";
# directory containing Analog COMPUTER output, for Report Magic & logtable.pl
my $outputpath = "$basedir/stat";
# destination for formatted (HTML) output -- both report directories and summary table
my $logtable = glob("~/bin/logtable.pl");
my $verbosity = 0;
my ($arg, $site, $sitepath, $path, @args);

while (scalar @ARGV) {
	$arg = shift @ARGV;
	if ($arg eq "-V") {
		print "logwrangler.pl, version $version\n";
		exit 0;
	}
	elsif ($arg eq "--version") {
		print "logwrangler.pl, version $version\n";
		exit 0;
	}
	elsif ($arg eq "-q") { $verbosity--; }
	elsif ($arg eq "-v") { $verbosity++; }
	else {
		print "$arg";
		die "Usage: logwrangler.pl [-Vqv] [--version]";
	}
}

if (defined $dnstran and defined $logs and -x $dnstran) {
	if ($verbosity > 0) { print "Running DNSTran against $logs.\n"; }
	@args = ("echo", "$dnstran", "@dnstranargs", "$logs");
	system (@args) == 0 or die "system @args failed: $?";
} elsif ($verbosity > 0) { warn "DNSTran unavailable; skipped.\n"; }

opendir (CONFIGPATH, $configpath) or die "$0: couldn't open $configpath: $1";
if ($verbosity > 0) { print "\nRunning Analog against $configpath/*\n"; }
foreach $site (sort(readdir(CONFIGPATH))) {
	if ($site =~ /^\./) { next; }
	if ($verbosity > 0) {
		printf "%26s: ", $site;
		@args = ("time", "$analog", "+g$configpath/$site");
		system (@args) == 0 or die "system @args failed: $?";
	}
	elsif ($verbosity == 0) {
		print "$site ";
		@args = ("$analog", "+g$configpath/$site");
		system (@args) == 0 or die "system @args failed: $?";
	}
	else {
		@args = ("$analog", "+g$configpath/$site");
		system (@args) == 0 or die "system @args failed: $?";
	}
}
if ($verbosity == 0) { print ("\n"); }

if (-x "$rmagic" and (opendir (DATAPATH, $datapath))) {
	if ($verbosity > 0) { print "\nRunning Report Magic against $configpath/*\n"; }
	foreach $site (sort(readdir(DATAPATH))) {
		if ($site =~ /^\./) { next; }
		if ($site =~ m{^(.+/)?([-a-z0-9.]+)}) { $sitepath = $2; }
		else {
			warn "Pattern match failed -- bad filename $site.\n";
			next;
		}
		if ($sitepath =~ m{^([-a-z0-9.]+)\.(txt|out|dat)}) { $sitepath = $1; }
			if ($verbosity > 0) { print "$site "; }
		@args = ("$rmagic", "-statistics-File_In=$datapath/$site", "-reports-File_Out=$outputpath/$sitepath/index.html");
		# if your rmagic.pl can't find rmagic.ini, you can add the full path as the second member of the list above
		system (@args) == 0 or die "system @args failed: $?";
	}
} elsif ($verbosity > 0) { warn "Couldn't run Report Magic; skipped; $!.\n"; }

if (open (LOGTABLE, "$logtable $datapath/* |") and open (LOGTABLEOUT, "> $outputpath/index.html")) {
	if ($verbosity > 0) { print "\nRunning logtable.pl against $datapath/*\n"; }
	print LOGTABLEOUT <LOGTABLE>; # generate the summary table
	close LOGTABLE;
	close LOGTABLEOUT;
} else { warn "Failed to run logtable.pl; skipping table generation: $!\n"; }

if ($verbosity > 0) { print "\n$0 complete: $outputpath.\n"; }
